/********************************************************* 
 * Bugblatter Map Voting Extension - V1.5                *
 *********************************************************
 *                                                       *
 * This plug-in is linux & win32 friendly.               *
 *                                                       *
 * Version 1.5:                                          *
 *                                                       *
 *  - Prevents nominating the current map, however it    *
 *    was selected.  Also prevents the current map being *
 *    used to fill the voting menu                       *
 *  - scores are now shown at map change reliably        *
 *  - bb_maps function implemented to list all maps      *
 *  - Intergated !2SX!'s antiadminkick plugin - enable   *
 *    it with bb_mapexecadminpass on                     *
 *                                                       *
 * Version 1.4:                                          *
 *                                                       *
 *  - Fixed bug in failing to increase mp_timelimit      *
 *    when the map was extended.  Introduced in 1.3.     *
 *  - Added verbosity & message location control.        *
 *  - Added reset command.                               *
 *  - Made all admin commands report back the config     *
 *  - Map names / say commands are case insensitive -    *
 *    nominating 'dust' and 'Dust' won't result in two   *
 *    nominations.                                       *
 *  - Prevents duplication menu entires if you nominate  *
 *    a map that would be used to fill empty menus       *
 *  - Added support for 'rockthevote'                    *
 *  - Handles errors if another plugin is using the      *
 *    menu system. postpones the vote for 1 minute       *
 *                                                       *
 *                                                       *
 * Version 1.3:                                          *
 *                                                       *
 *  - Fixed bug in setting mp_timelimit 60 times too     *
 *    high.                                              *
 *                                                       *
 * Version 1.2:                                          *
 *                                                       *
 *  - Fixed bug in saving to maps.ini                    *
 *                                                       *
 * Version 1.0 - Initial Release                         *
 *********************************************************
 */

/* TODO:
 *   - Prevent votes in the first 20 seconds of a round.
 */


#include <core>
#include <console>
#include <string>
#include <admin>
#include <adminlib>

/* You can change these if you really want, but all the really
 * useful ones can be set from the new admin commands in the console*/

/* Maximum number of maps that will be included in rotation */
#define MAX_MAPS 100
/* Longest allowed map filename */
#define MAX_MAP_LENGTH 32
/* Time after vote finishes before map change */
#define MAP_CHANGE_DELAY 20

/* Default Number of maps in the nominations menu */
#define DEFAULT_NOMINATIONS 4
/* Maximum Number of maps in the nominations menu */
#define MIN_NOMINATIONS 2
#define MAX_NOMINATIONS 8

/* Amount of time to extend the map if that wins the vote (minutes) */
#define DEFAULT_EXTEND_TIME 10
/* Maximum allowed extend time */
#define MIN_EXTEND_TIME 5
#define MAX_EXTEND_TIME 60

/* Default number of previous maps that can't be nominated */
#define DEFAULT_BAN_LAST 3
/* Maximum number of previous maps that can't be nominated */
#define MIN_BAN_LAST 0
#define MAX_BAN_LAST 10

/* Default maximum number of times the map can be extended */
#define DEFAULT_MAX_EXTENDS 3
/* Maxmimum maximum number of times the map can be extended */
#define MIN_MAX_EXTENDS 0
#define MAX_MAX_EXTENDS 1000

/* Default map verbosity */
#define DEFAULT_VERBOSITY 255
#define MIN_VERBOSITY 0
#define MAX_VERBOSITY 255

#define DEFAULT_ROCKTHEVOTE 10
#define MIN_ROCKTHEVOTE 0
#define MAX_ROCKTHEVOTE 1000

#define DEFAULT_MESSAGELOCATION 241
#define MIN_MESSAGELOCATION 0
#define MAX_MESSAGELOCATION 255

#define DEFAULT_MESSAGEFREQUENCY 2
#define MIN_MESSAGEFREQUENCY 1
#define MAX_MESSAGEFREQUENCY 1000

#define DEFAULT_EXECADMINPASS 0

#define FLAG_WELCOMEMESSAGE 1
#define FLAG_AVAILABLEMAPS 2
#define FLAG_NOMINATEDMAPS 4
#define FLAG_NOMINATION 8
#define FLAG_COUNTDOWNMESSAGE 16
#define FLAG_COUNTDOWNTALK 32
#define FLAG_VOTINGINPROGRESS 64
#define FLAG_WINNEROFVOTE 128



/* **************************************************************************************** */

new g_AllMaps[MAX_MAPS][MAX_MAP_LENGTH];                    /* Array contains map names */
new g_AllMapsShort[MAX_MAPS][MAX_MAP_LENGTH];               /* Names with the underscore bit gone */
new g_AllMapsSize=0;                                        /* Number of slots filled in g_AllMaps */
new g_NominatedMaps[MAX_NOMINATIONS][MAX_MAP_LENGTH];       /* Array containing nominations */
new g_Nominators[MAX_NOMINATIONS+2];                        /* Array contain user IDs of nominators */
new g_ThisMenuSize = DEFAULT_NOMINATIONS;		    /* Number of maps in the current nominations menu */
new g_NextMenuSize=DEFAULT_NOMINATIONS;                     /* Number of maps in the next nominations menu */
new g_VoteActive=0;					    /* 1 = Voting process has begun / 0 = Nomiations allowed */
new g_NextMap[MAX_MAP_LENGTH];				    /* Name of map that won the vote, if extend didn't win */
new g_InfoToggle=0;					    /* 1 = Display nominations next / 0 =  Display available maps next */
new g_InfoPosition=0;					    /* Index in g_AllMaps to read next available maps list from */
new g_InfoCount=DEFAULT_MESSAGEFREQUENCY;                   /* Number of minutes until next announcement */
new g_NumExtends=0;                                         /* Number of times the current map has been extended */
new g_MaxExtends=DEFAULT_MAX_EXTENDS;			    /* Number of times the current map *can* be extended */
new g_ExtendTime=DEFAULT_EXTEND_TIME;			    /* Number of minutes a map extension adds to the clock */
new g_BanMaps=DEFAULT_BAN_LAST;				    /* Number of previous maps that can't be nominated */
new g_RoundTime=1200;					    /* Time between votes. Overridden by mp_timelimit in server.cfg */
new g_Enabled=1;					    /* 1 = Voting enabled / 0 = No voting allowed - use normal map rotation */
new g_Verbosity=DEFAULT_VERBOSITY			    /* Series of flags saying what to announce */
new g_MessageLocation=DEFAULT_MESSAGELOCATION		    /* Flags saying where to display message */
new g_MessageFrequency=DEFAULT_MESSAGEFREQUENCY             /* Time between announcements */
new g_RockTheVote=DEFAULT_ROCKTHEVOTE                       /* Time before rock the vote is allowed */
new g_Rocked=0;                                             /* True if this rock was rocked */
new g_ExecAdminPass=DEFAULT_EXECADMINPASS;		    /* True if players with reserved names/spots get adminpass exec'd on map change */

/* Prototype is necessary for these functions  */
forward BuildLine(msg[],txt[],extra,&linelen);
forward IsBanned(MapName[]);
forward SayAll(Msg[],Flag,Duration,Red,Green,Blue);

/* **************************************************************************** */
/* Initialisation function */


public plugin_init() {
 
  /* Register callbacks with admin mod */ 
  plugin_registerinfo("Bugblatter's Map Vote 1.5","Advanced map voter","1.5");
  plugin_registercmd("say","SayCommand",ACCESS_ALL);
  plugin_registercmd("admin_vote_map","AdminVoteMap",ACCESS_ALL,
                     "admin_vote_map not available");
  plugin_registercmd("votemap","OriginalVoteMap",ACCESS_ALL,
                     "votemap not available");

  plugin_registercmd("bb_mapvote","BBMapVote",ACCESS_MAP,
                     "bb_mapvote on|off|now");	
  plugin_registercmd("bb_mapcount","BBMapCount",ACCESS_MAP,
                     "bb_mapcount n - set number of maps in menu");	
  plugin_registercmd("bb_mapextend","BBMapExtend",ACCESS_MAP,
                     "bb_mapextend n - set maximum number of times you can for to stay on same map");	
  plugin_registercmd("bb_mapextendtime","BBMapExtendTm",ACCESS_MAP,
                     "bb_mapextendtime n - set number of minutes an extension lasts");	
  plugin_registercmd("bb_mapbanlast","BBMapBanLast",ACCESS_MAP,
                     "bb_mapbanlast n - set number of recently played maps to ban");	
  plugin_registercmd("bb_mapverbosity","BBMapVerbosity",ACCESS_MAP,
                     "bb_mapverbosity n - see bugblatter plugins manual for value of n");	
  plugin_registercmd("bb_mapmsglocation","BBMapMsgLocation",ACCESS_MAP,
                     "bb_mapmsglocation n - see bugblatter plugins manual for value of n");	
  plugin_registercmd("bb_mapmsgfrequency","BBMapMsgFrequency",ACCESS_MAP,
                     "bb_mapmsgfrequency n - number of minutes between announcements");	
  plugin_registercmd("bb_mapreset","BBMapReset",ACCESS_MAP,
                     "bb_mapreset - sets all map voting settings to defaults");	  
  plugin_registercmd("bb_maprockthevote","BBMapRockTheVote",ACCESS_MAP,
		     "bb_maprockthevote n - number of minutes before rockthevote can be used");
  plugin_registercmd("bb_mapadminpass","BBMapAdminPass",ACCESS_CONFIG,
		     "bb_mapadminpass on|off - enables execing adminpass.cfg on map change");
  plugin_registercmd("bb_maps","BBMaps",ACCESS_ALL,
		     "bb_maps - lists all maps available for voting");

  /* g_Nominators array has 2 unused extra slots to
   * simplify the map listing function. This sets them
   * to 0, where they will stay */
  g_Nominators[MAX_NOMINATIONS]=0;
  g_Nominators[MAX_NOMINATIONS+1]=0;

    
  /* Read saved configuration from vault */
  readvaultnum("bb_mapvote",g_Enabled,1,0,1);
  readvaultnum("bb_mapexecadminpass",g_ExecAdminPass,DEFAULT_EXECADMINPASS,0,1);
  readvaultnum("bb_mapcount", g_NextMenuSize,DEFAULT_NOMINATIONS,MIN_NOMINATIONS,MAX_NOMINATIONS);
  readvaultnum("bb_mapextend",g_MaxExtends,DEFAULT_MAX_EXTENDS,MIN_MAX_EXTENDS,MAX_MAX_EXTENDS);
  readvaultnum("bb_mapextendtime",g_ExtendTime,DEFAULT_EXTEND_TIME,MIN_EXTEND_TIME,MAX_EXTEND_TIME);
  readvaultnum("bb_mapbanlast",g_BanMaps, DEFAULT_BAN_LAST,MIN_BAN_LAST,MAX_BAN_LAST);
  readvaultnum("bb_mapverbosity",g_Verbosity, DEFAULT_VERBOSITY,MIN_VERBOSITY,MAX_VERBOSITY);
  readvaultnum("bb_mapmsglocation",g_MessageLocation, DEFAULT_MESSAGELOCATION,MIN_MESSAGELOCATION,MAX_MESSAGELOCATION);
  readvaultnum("bb_mapmsgfrequency",g_MessageFrequency, DEFAULT_MESSAGEFREQUENCY,MIN_MESSAGEFREQUENCY,MAX_MESSAGEFREQUENCY);
  readvaultnum("bb_maprockthevote",g_RockTheVote, DEFAULT_ROCKTHEVOTE,MIN_ROCKTHEVOTE,MAX_ROCKTHEVOTE);
  
  
  ReadMapList();
  /* Move the first 2 maps in the list to bb_mapbanlast places from the end
   * This prevents the same maps being used to pad the menu over and over if
   * there aren't enough nominations */
  new mapname[MAX_MAP_LENGTH];
  strcpy(mapname,g_AllMaps[0],MAX_MAP_LENGTH);
  MoveMapToEnd(mapname,1);
  strcpy(mapname,g_AllMaps[0],MAX_MAP_LENGTH);
  MoveMapToEnd(mapname,1);
  SaveMapList();

  /* Prime the voting menu */  
  new i;
  for(i=0;i<MAX_NOMINATIONS;i++) {
    g_Nominators[i]=0;
    g_NominatedMaps[i]="<None>";
  }
  g_ThisMenuSize = g_NextMenuSize;

  
  /* Prevent built-in map rotation getting in the way */
  g_RoundTime = getvar("mp_timelimit") * 60;
  SetMapTime(g_RoundTime+300);

  /* Start the announcement / vote start timer */
  set_timer("Timer",60,1,"");
  return PLUGIN_CONTINUE;
}


public plugin_connect(HLUserName, HLIP, UserIndex) {
  set_timer("PerformConnect",60,1);
  return PLUGIN_CONTINUE;
}

public plugin_disconnect(HLUserName, UserIndex) {
  return PLUGIN_CONTINUE;
}

public PerformConnect(Timer,Repeat,HLName,HLParam) {
  new User[MAX_NAME_LENGTH];
  convert_string(HLName,User,MAX_NAME_LENGTH);
  if ((g_Verbosity & FLAG_WELCOMEMESSAGE) >0) {
    if ((g_MessageLocation & FLAG_WELCOMEMESSAGE) > 0) {
      messageex(User,"This server runs map voting software.",4);
    }
    else {
      messageex(User,"This server runs map voting software.",2);
    }
  }
  return PLUGIN_CONTINUE;
}


/* **************************************************************************** */
/* Administrator Command handlers */

public AdminVoteMap(HLCommand,HLData,HLUserName,UserIndex) {
  selfmessage("admin_vote_map is not in use on this server.");
  selfmessage("say ^"nextmap^" for more information on voting.");
  return PLUGIN_HANDLED;
}

public OriginalVoteMap(HLCommand,HLData,HLUserName,UserIndex) {
  selfmessage("votemap is not in use on this server.");
  selfmessage("say ^"nextmap^" for more information on voting.");
  return PLUGIN_HANDLED;
}

public BBMapVote(HLCommand,HLData,HLUserName,UserIndex) {
  new Data[MAX_DATA_LENGTH];
  convert_string(HLData,Data,MAX_DATA_LENGTH);
  
  if (strcasecmp(Data,"on")==0) {
    g_Enabled = 1;
    set_timer("Timer",60,1,"");    
  }
  else if(strcasecmp(Data,"off")==0) {
    g_Enabled = 0;
    SetMapTime(g_RoundTime);
  }
  else if (strcasecmp(Data,"now")==0) {
    g_Enabled = 1;
    DoStartVote(1);
  }
  else {
    selfmessage("usage: bb_mapvote on | off | now");
  }

  writevaultnum("bb_mapvote",g_Enabled);
  return ShowConfig();

} 

public BBMapCount(HLCommand,HLData,HLUserName,UserIndex) {
  if (readHLnum(HLData,g_NextMenuSize,5,2,MAX_NOMINATIONS)) {
	writevaultnum("bb_mapcount",g_NextMenuSize);
  }
  return ShowConfig();
}

public BBMapExtend(HLCommand,HLData,HLUserName,UserIndex) {
  if (readHLnum(HLData,g_MaxExtends,DEFAULT_MAX_EXTENDS,MIN_MAX_EXTENDS,MAX_MAX_EXTENDS)) {
    writevaultnum("bb_maxextends",g_MaxExtends);
  }
  return ShowConfig();
}

public BBMapExtendTm(HLCommand,HLData,HLUserName,UserIndex) {
  if (readHLnum(HLData,g_ExtendTime,DEFAULT_EXTEND_TIME,MIN_EXTEND_TIME,MAX_EXTEND_TIME)) {
    writevaultnum("bb_mapextendtime",g_ExtendTime);
  }
  return ShowConfig();
}

public BBMapBanLast(HLCommand,HLData,HLUserName,UserIndex) {
  if (readHLnum(HLData,g_BanMaps,DEFAULT_BAN_LAST,MIN_BAN_LAST,MAX_BAN_LAST)) {
    writevaultnum("bb_mapbanlast",g_BanMaps);
  }
  return ShowConfig();
}

public BBMapVerbosity(HLCommand,HLData,HLUserName,UserIndex) {
  if (readHLnum(HLData,g_Verbosity,DEFAULT_VERBOSITY,MIN_VERBOSITY,MAX_VERBOSITY)) {
    writevaultnum("bb_mapverbosity",g_Verbosity);
  }
  return ShowConfig();
}

public BBMapMsgLocation(HLCommand,HLData,HLUserName,UserIndex) {
  if (readHLnum(HLData,g_MessageLocation,DEFAULT_MESSAGELOCATION,MIN_MESSAGELOCATION,MAX_MESSAGELOCATION)) {
    writevaultnum("bb_mapmsglocation",g_MessageLocation);
  }
  return ShowConfig();
}

public BBMapMsgFrequency(HLCommand,HLData,HLUserName,UserIndex) {
  if (readHLnum(HLData,g_MessageFrequency,DEFAULT_MESSAGEFREQUENCY,MIN_MESSAGEFREQUENCY,MAX_MESSAGEFREQUENCY)) {
    writevaultnum("bb_mapmsgfrequency",g_MessageFrequency);
    if (g_InfoCount+1>g_MessageFrequency) {
      g_InfoCount = g_MessageFrequency-1;
    }
  }
  return ShowConfig();
}

public BBMapRockTheVote(HLCommand,HLData,HLUserName,UserIndex) {
  if (readHLnum(HLData,g_RockTheVote,DEFAULT_ROCKTHEVOTE,MIN_ROCKTHEVOTE,MAX_ROCKTHEVOTE)) {
    writevaultnum("bb_maprockthevote",g_RockTheVote);
  }
  return ShowConfig();
}

public BBMaps(HLCommand,HLData,HLUserName,UserIndex) {
  ShowMapList();
  return PLUGIN_CONTINUE;
}

public BBMapAdminPass(HLCommand,HLData,HLUserName,UserIndex) {
  new Data[MAX_DATA_LENGTH];
  convert_string(HLData,Data,MAX_DATA_LENGTH);
  
  if (strcasecmp(Data,"on")==0) {
    g_ExecAdminPass = 1;
  }
  else if(strcasecmp(Data,"off")==0) {
    g_ExecAdminPass = 0;
  }

  writevaultnum("bb_mapexecadminpass",g_ExecAdminPass);
  return ShowConfig();

} 

public BBMapReset(HLCommand,HLData,HLUserName,UserIndex) {
  g_InfoCount=DEFAULT_MESSAGEFREQUENCY;
  g_MaxExtends=DEFAULT_MAX_EXTENDS;
  g_ExtendTime=DEFAULT_EXTEND_TIME;
  g_BanMaps=DEFAULT_BAN_LAST;
  g_Enabled=1;
  g_Verbosity=DEFAULT_VERBOSITY;
  g_MessageLocation=DEFAULT_MESSAGELOCATION;
  g_MessageFrequency=DEFAULT_MESSAGEFREQUENCY;
  g_RockTheVote = DEFAULT_ROCKTHEVOTE;
  g_NextMenuSize = DEFAULT_NOMINATIONS;
  g_ExecAdminPass=DEFAULT_EXECADMINPASS;

  writevaultnum("bb_mapvote",g_Enabled);
  writevaultnum("bb_mapcount",g_NextMenuSize);
  writevaultnum("bb_maxextends",g_MaxExtends);
  writevaultnum("bb_mapextendtime",g_ExtendTime);
  writevaultnum("bb_mapbanlast",g_BanMaps);
  writevaultnum("bb_mapverbosity",g_Verbosity);
  writevaultnum("bb_mapmsglocation",g_MessageLocation);
  writevaultnum("bb_mapmsgfrequency",g_MessageFrequency);
  writevaultnum("bb_maprockthevote",g_RockTheVote);
  writevaultnum("bb_mapexecadminpass",g_ExecAdminPass);
  return ShowConfig();
}

public ShowConfig() {  
  selfmessage("Bugblatter map voting configuration:");
  if (g_Enabled) {
    selfmessage(" - Voting is current ENABLED (bb_mapvote on)");
  }
  else {
    selfmessage(" - Voting is current DISABLED (bb_mapvote off)");
  }    
  ShowLine(" - Players may nominate %i different maps, effective after this vote. (bb_mapcount)",g_NextMenuSize);
  ShowLine(" - Players may vote to extend the map %i times before the option is removed. (bb_mapextend)",g_MaxExtends);
  ShowLine(" - Maps extensions will last for %i minutes. (bb_mapextendtime)",g_ExtendTime);
  ShowLine(" - Players are be unable to nominate the last %i maps played. (bb_mapbanlast)",g_BanMaps);
  ShowLine(" - Message verbosity is set to %i - see manual for more details. (bb_mapverbosity)",g_Verbosity);
  ShowLine(" - Message location is set to %i - see manual for more details. (bb_mapmsglocation)",g_MessageLocation);
  ShowLine(" - Announcement messages are displayed every %i miuntes. (bb_mapmsgfrequency)",g_MessageFrequency);
  ShowLine(" - Players can rock the vote after %i minutes. (bb_maprockthevote)",g_RockTheVote);

  if (g_ExecAdminPass) {
    selfmessage(" - Adminpass.cfg will be exec'd on map change for reserved users");
  }
  else {
    selfmessage(" - Adminpass.cfg will not be exec'd on map change");
  }    
  return PLUGIN_HANDLED;
}


/* **************************************************************************** */
/* Event handlers */




/* Invoked by adminmod when a user says anything *
 * Checks for "commands" in the message */
public SayCommand(HLCommand,HLData,HLUserName,UserIndex) {
  new Data[MAX_DATA_LENGTH];
  new UserName[MAX_NAME_LENGTH];
  convert_string(HLData,Data,MAX_DATA_LENGTH);
  convert_string(HLUserName,UserName,MAX_NAME_LENGTH);

  strstripquotes(Data);
  
  if (strmatch(Data,"nominations",11)==1) {
    ReportNominations(UserName);
    return PLUGIN_CONTINUE;
  }

  if (strmatch(Data,"nominate ",9)==1) {
    new i;
    new lenData = strlen(Data);
    for(i=9;i<=lenData;i++) {
      Data[i-9] = Data[i];
    }
    Data[i-9] = NULL_CHAR;
    Nominate(Data,UserName,UserIndex,1);
    return PLUGIN_CONTINUE;
  }

  if (strmatch(Data,"vote ",5)==1) {
    new i;
    new lenData = strlen(Data);
    for(i=5;i<=lenData;i++) {
      Data[i-5] = Data[i];
    }
    Data[i-5] = NULL_CHAR;
    Nominate(Data,UserName,UserIndex,1);
    return PLUGIN_CONTINUE;
  }

  /* Take over admin_chat's role */
  if (strcasecmp(Data, "timeleft")==0) {
    SayTimeleft();
    return PLUGIN_CONTINUE;
  } else if (strcasecmp(Data, "version")==0) {
    say("Type 'admin_version' in the console for version information.");
    return PLUGIN_CONTINUE;
  } else if (strcasecmp(Data, "nextmap")==0) {
    SayNextMap();
    return PLUGIN_CONTINUE;
  } else if (strcasecmp(Data, "currentmap")==0) {
    SayCurrentMap();
    return PLUGIN_CONTINUE;
  } else if (strcasecmp(Data, "listmaps")==0) {
    ReportMapsAvailable(UserName);
    return PLUGIN_CONTINUE;
  } else if (strcasecmp(Data, "rockthevote")==0) {
    RockTheVote(UserName);
    return PLUGIN_CONTINUE;
  } else if (strcasecmp(Data, "bb_maps")==0) {
    messageex(UserName,"Map list displayed in your console - press ~ or `",2);
    messageex(UserName,"Please enter this command in the console in future",2);
    ShowMapList();
  }

  Nominate(Data,UserName,UserIndex,0);
  return PLUGIN_CONTINUE;
}

/* Handles the "nominate" command - invoked from SayCommand */
Nominate(MapName[],UserName[],UserIndex,ShowErrors) {
  new MapIndex = LookupMap(MapName);
  new msg[MAX_TEXT_LENGTH];
  new linelen =0;
  msg="";

  if (g_Enabled ==0) {
    return 0;
  }

  if (g_VoteActive == 1) {
    if (ShowErrors) {
      messageex(UserName,"Voting is in progress.^nYou cannot nominate at this time.",4);
    }
    return 0;
  }
  
  if (MapIndex == -1) {
    if (ShowErrors) {
      msg = "Map ";
      strcat(msg, MapName,  MAX_TEXT_LENGTH);
      strcat(msg, " was not found on this server.",  MAX_TEXT_LENGTH);
      messageex(UserName,msg,4);
    }
    return 0;
  }

  if (IsNominated(MapName)==1) {
    BuildLine(msg,"Map ",0,linelen);
    BuildLine(msg, MapName, 0,linelen);
    BuildLine(msg, " has already been nominated.",0,linelen);
    messageex(UserName,msg,4);
    return 1;
  }

  if (IsBanned(MapName)==1) {
    BuildLine(msg,"You cannot nominate Map ",0,linelen);
    BuildLine(msg,MapName,0,linelen);
    if (g_BanMaps == 1) {
      BuildLine(msg,"^nbecause it was the last map played.",0,linelen);
    }
    else {
      new strBanMaps[10];
      numtostr(g_BanMaps,strBanMaps);
      BuildLine(msg, "^nbecause it was one of the last ",0,linelen);
      BuildLine(msg, strBanMaps,0,linelen);
      BuildLine(msg, " maps played.",0,linelen);
    }
    messageex(UserName,msg,4);
    return 1;
  }

  new NominationIndex = LookupFreeSlot(UserIndex);
  if (NominationIndex ==-1) {
    messageex(UserName,"Sorry - The maximum number of nominations^nhas already been reached.",4);
    return 0;
  }

  strcpy(g_NominatedMaps[NominationIndex],MapName,MAX_MAP_LENGTH);
  g_Nominators[NominationIndex]=UserIndex;
  BuildLine(msg,UserName,0,linelen);
  BuildLine(msg," has nominated ",0,linelen);
  BuildLine(msg,MapName,0,linelen);
  BuildLine(msg,".",0,linelen);

  if ((g_MessageLocation & FLAG_NOMINATION)>0) {
    strcat(msg,"^nTo nominate a map, say: nominate MAPNAME",MAX_TEXT_LENGTH);
    SayAll(msg,FLAG_NOMINATION,15,255,0,255);
  }
  else {
    SayAll(msg,FLAG_NOMINATION,15,255,0,255);
    SayAll("To nominate a map, say: nominate MAPNAME",FLAG_NOMINATION,15,255,0,255);
  }
  return 1;

}

/* Called every 10 seconds *
 * Detects when a map change is about to happen and 
 * starts a menu driven vote */
public Timer(Dummy) {

  if (g_Enabled ==0) {
    return 0;
  }

  if (g_VoteActive == 1) {
    /* Timer shouldn't be running if a vote is active, but
     * check anyway for safety */
    return 0;
  }

  /* Find out how much time is left */
  if (maptime(0,0) >= g_RoundTime) {
    DoInformVote();
  }
  else {
    if (g_InfoCount > 0) {
      g_InfoCount--;
    }
    else {
      g_InfoCount = g_MessageFrequency-1;
      g_InfoToggle = 1 - g_InfoToggle;
      if ((g_InfoToggle==0) || ((g_Verbosity & FLAG_AVAILABLEMAPS)==0)) {
	/* Report to the users on the current nomination status */
	if ((g_Verbosity & FLAG_NOMINATEDMAPS)>0) {
	  ReportNominations("");
	}
      }
      else {
	ReportMapsAvailable("");
      }
    }
    set_timer("Timer",60,1,"");
    
  }
  
  return 0;
}

DoInformVote() {
  if ((g_Verbosity & FLAG_COUNTDOWNTALK) > 0) {
    execute_all("speak ^"fvox/twenty seconds remaining^"");
  }
  SayAll("Voting is about to begin",FLAG_COUNTDOWNMESSAGE, 15,255,0,0);
  set_timer("VSTimer",10,1,"");
}

public VSTimer() {
  if ((g_Verbosity & FLAG_COUNTDOWNTALK) > 0) {
    execute_all("speak ^"fvox/ten seconds remaining^"");
  }
  SayAll("Voting is about to begin",FLAG_COUNTDOWNMESSAGE,15,255,0,0);
  set_timer("VSTimer2",10,1,"");
}

public VSTimer2() {
  DoStartVote(0);
}

public RockTheVote(UserName[]) {
  new Verb[10] = "minutes";
  new Text[MAX_TEXT_LENGTH];

  if (g_VoteActive) {
    messageex(UserName,"Voting already in progress.",2);
  }

  if (g_RockTheVote >= g_RoundTime) {
    messageex(UserName,"Administrator has not enabled 'vote rocking' on this server",2);
    return 0;
  }
  
  
  new rocktime = (g_RockTheVote*60) - maptime(0,0);
  if (rocktime < 0) {
    DoStartVote(1);
    return 0;
  }
  
  
  rocktime /= 60;
  rocktime += 2;	
  if (rocktime < 1) rocktime =1;
  if (rocktime ==1) Verb = "minute";
  
  snprintf(Text, MAX_TEXT_LENGTH, "Vote cannot be rocked for another %i %s.", rocktime,Verb);    
  messageex(UserName,Text,2);
  return 0;
}

DoStartVote(Rocked) {
  new i=0;
  new j=0;
  
  /*Time to start a vote */
  g_VoteActive=1;

  new Current[MAX_MAP_LENGTH];
  currentmap(Current,MAX_MAP_LENGTH);
 
  /*Pad any free slots with top maps */
  for(i=0;i<g_ThisMenuSize;i++) {
    if (g_Nominators[i] ==0) {
      while ((IsNominated(g_AllMaps[j])==1) ||
             (strcasecmp(g_AllMaps[j],Current)==0)) {
	j++;
      }

      g_NominatedMaps[i] = g_AllMaps[j];
      j++;
    }
  }

  /* No point in voting if no-one connected */
  if (playercount() == 0) {
    return 0;
  }

  new fOK;  
  g_Rocked=Rocked;
  if ((g_NumExtends<g_MaxExtends) || (g_Rocked>0)) {
    switch(g_ThisMenuSize) {
    case 2:
      fOK=vote("What map would you like to play next?","Stay on this map",
	       g_NominatedMaps[0],g_NominatedMaps[1],
	       "VoteComplete","");      
    case 3:
      fOK=vote("What map would you like to play next?","Stay on this map",
	       g_NominatedMaps[0],g_NominatedMaps[1],g_NominatedMaps[2],
	       "VoteComplete","");
    case 4:
      fOK=vote("What map would you like to play next?","Stay on this map",
	       g_NominatedMaps[0],g_NominatedMaps[1],g_NominatedMaps[2],g_NominatedMaps[3],
	       "VoteComplete","");
    case 5:
      fOK=vote("What map would you like to play next?","Stay on this map",
	       g_NominatedMaps[0],g_NominatedMaps[1],g_NominatedMaps[2],g_NominatedMaps[3],
	       g_NominatedMaps[4],
	       "VoteComplete","");
    case 6:
      fOK=vote("What map would you like to play next?","Stay on this map",
	       g_NominatedMaps[0],g_NominatedMaps[1],g_NominatedMaps[2],g_NominatedMaps[3],
	       g_NominatedMaps[4],g_NominatedMaps[5],
	       "VoteComplete","");
    case 7:
      fOK=vote("What map would you like to play next?","Stay on this map",
	       g_NominatedMaps[0],g_NominatedMaps[1],g_NominatedMaps[2],g_NominatedMaps[3],
	       g_NominatedMaps[4],g_NominatedMaps[5],g_NominatedMaps[6],
	       "VoteComplete","");
    case 8:
      fOK=vote("What map would you like to play next?","Stay on this map",
	       g_NominatedMaps[0],g_NominatedMaps[1],g_NominatedMaps[2],g_NominatedMaps[3],
	       g_NominatedMaps[4],g_NominatedMaps[5],g_NominatedMaps[6],g_NominatedMaps[7],
	       "VoteComplete","");
    }
  }
  else {
    switch(g_ThisMenuSize) {
    case 2:
      fOK=vote("What map would you like to play next?",
	       g_NominatedMaps[0],g_NominatedMaps[1],
	       "VoteComplete","");      
    case 3:
      fOK=vote("What map would you like to play next?",
	       g_NominatedMaps[0],g_NominatedMaps[1],g_NominatedMaps[2],
	       "VoteComplete","");
    case 4:
      fOK=vote("What map would you like to play next?",
	       g_NominatedMaps[0],g_NominatedMaps[1],g_NominatedMaps[2],g_NominatedMaps[3],
	       "VoteComplete","");
    case 5:
      fOK=vote("What map would you like to play next?",
	       g_NominatedMaps[0],g_NominatedMaps[1],g_NominatedMaps[2],g_NominatedMaps[3],
	       g_NominatedMaps[4],
	       "VoteComplete","");
    case 6:
      fOK=vote("What map would you like to play next?",
	       g_NominatedMaps[0],g_NominatedMaps[1],g_NominatedMaps[2],g_NominatedMaps[3],
	       g_NominatedMaps[4],g_NominatedMaps[5],
	       "VoteComplete","");
    case 7:
      fOK=vote("What map would you like to play next?",
	       g_NominatedMaps[0],g_NominatedMaps[1],g_NominatedMaps[2],g_NominatedMaps[3],
	       g_NominatedMaps[4],g_NominatedMaps[5],g_NominatedMaps[6],
	       "VoteComplete","");
    case 8:
      fOK=vote("What map would you like to play next?",
	       g_NominatedMaps[0],g_NominatedMaps[1],g_NominatedMaps[2],g_NominatedMaps[3],
	       g_NominatedMaps[4],g_NominatedMaps[5],g_NominatedMaps[6],g_NominatedMaps[7],
	       "VoteComplete","");
    }
  }
  
  if (fOK) {
    SayAll("Map voting in progress...",FLAG_VOTINGINPROGRESS,30,255,0,0);
  }
  else {
    g_VoteActive=0;
    if (Rocked) {
      say("Attempt to rock the vote failed.");
      say("Another type of vote is in progress, or recently finished.");
    }
    else {
      SayAll("Error starting vote. ^nWill try again in 1 minute",FLAG_VOTINGINPROGRESS,10,255,0,0);
      set_timer("Timer",60,1,"");
    }
  }
  

  return 0;
}

/* Invoked by adminmod when a menu driven vote is complete */
/* es vote results and changes map */
public VoteComplete(Winner,HLData,VoteCount,UserCount) {
  new msg[MAX_TEXT_LENGTH];
  new strVotes[10];
  new strUsers[10];
  numtostr(VoteCount,strVotes);
  numtostr(UserCount,strUsers);
  
  if (g_Enabled ==0) {
    return 0;
  }
  
  msg = "Voting Complete^n";

  if ((Winner < 2) && ((g_NumExtends < g_MaxExtends) || g_Rocked)) {
    strcat(msg,"Remaining on this map won with ",MAX_TEXT_LENGTH);
    strcat(msg,strVotes,MAX_TEXT_LENGTH);
    strcat(msg," of ",MAX_TEXT_LENGTH);
    strcat(msg,strUsers,MAX_TEXT_LENGTH);
    strcat(msg," votes.^nNominations will be kept for next vote.",MAX_TEXT_LENGTH);

    if (g_Rocked == 0) {
      g_NumExtends++;
      g_RoundTime = g_RoundTime + (g_ExtendTime *60);
      SetMapTime(g_RoundTime + 300);
    }
    set_timer("Timer",60,1,"");    
    g_VoteActive = 0;
  }
  else {
    new MapIndex = Winner-2;
    if (g_NumExtends==g_MaxExtends) {
      MapIndex = Winner-1;
    }
    g_NumExtends=0;
    if (valid_map(g_NominatedMaps[MapIndex]) !=1) {
      strcat(msg,"Unfortunately the chosen map is no longer available.",MAX_TEXT_LENGTH);
    }
    else {
      strcat(msg,g_NominatedMaps[MapIndex],MAX_TEXT_LENGTH);
      strcat(msg," won with ",MAX_TEXT_LENGTH);
      strcat(msg,strVotes,MAX_TEXT_LENGTH);
      strcat(msg," of ",MAX_TEXT_LENGTH);
      strcat(msg,strUsers,MAX_TEXT_LENGTH);
      strcat(msg," votes.",MAX_TEXT_LENGTH);
      strcpy(g_NextMap,g_NominatedMaps[MapIndex],MAX_MAP_LENGTH);
      set_timer("DoChangeMap",MAP_CHANGE_DELAY,1);
      MoveMapToEnd(g_NextMap,0);
      SaveMapList();
    }
  }
  
  SayAll(msg,FLAG_WINNEROFVOTE,20,255,0,0);
  return 0;
}

/* Invoked by a timer from VoteComplete.
 * Actually performs the map change after the users
 * have a chance to read the message that it is
 * changing */
public DoChangeMap(Dummy)
{
  if (g_Enabled ==0) {
    return 0;
  }

  if (g_ExecAdminPass == 1) {
    new maxplayers = maxplayercount();
    new Name[MAX_NAME_LENGTH];
    new i=0;
    new SessionID=0;
    new WONID=0;
    new Team=0;

    for (i=1; i<=maxplayers; i++) {
      if (playerinfo(i,Name,MAX_NAME_LENGTH,SessionID,WONID,Team)==1) { 
	if ((access(ACCESS_RESERVE_SPOT,Name) == 1) || (access(ACCESS_RESERVE_NICK,Name) == 1)) {
	  execclient(Name, "exec adminpass.cfg");
	}
      }
    }
  }

  /* Delay the call to changelevel by 1 second to ensure 
     showscores takes effect */
  execute_all("+showscores");
  set_timer("DoChangeMap2",1,1,"");
  return 0;
}

public DoChangeMap2()
{
  changelevel(g_NextMap);
  return 0;
}


/* **************************************************************************** */
/* Borrowed from plugin_chat and modified */

SayCurrentMap() {
  if (g_Enabled) {
    return SayMapInfo();
  }
  
  new Text[MAX_TEXT_LENGTH];
  new CurrentMap[MAX_NAME_LENGTH];
  
  currentmap(CurrentMap,MAX_NAME_LENGTH);
  snprintf(Text, MAX_TEXT_LENGTH, "The current map is: %s", CurrentMap);
  say(Text);

  return 0;
}

SayTimeleft() {
  if (g_Enabled) {
    return SayMapInfo();
  }

  new Text[MAX_TEXT_LENGTH];
  new Verb[] = "minutes";
  new MapName[MAX_MAP_LENGTH];
  new Seconds = g_RoundTime - maptime(0,0);
  Seconds /= 60;	
  if (Seconds < 1) Seconds =1;
  if (Seconds ==1) Verb = "minute";

  currentmap(MapName,MAX_MAP_LENGTH);    
  snprintf(Text, MAX_TEXT_LENGTH, "Time remaining on map %s: %i %s.", MapName,Seconds,Verb);    
    
  say(Text);
  return 0;
}

SayNextMap() {
  if (g_Enabled) {
    return SayMapInfo();
  }

  new Text[MAX_TEXT_LENGTH];
  new MapName[MAX_MAP_LENGTH];

  nextmap(MapName,MAX_MAP_LENGTH);
  snprintf(Text, MAX_TEXT_LENGTH, "The next map will be: %s.", MapName);
  say(Text);
  return 0;
}

SayMapInfo() {
  new Text[MAX_TEXT_LENGTH];
  new Verb[] = "minutes";
  new CurrentMapName[MAX_MAP_LENGTH];
  new Seconds = g_RoundTime - maptime(0,0);
  Seconds /= 60;
  Seconds += 2;	
  if (Seconds < 1) Seconds =1;
  if (Seconds ==1) Verb = "minute";

  if (g_VoteActive ==1) {
    if (strlen(g_NextMap)>0) {
      snprintf(Text, MAX_TEXT_LENGTH, "Players have voted for %s as the next map.",g_NextMap);
      say(Text);
      say("The map will be changed soon.")
    }
    else {
      say("Voting is in progress for the next map. Result will be announced soon.");
    }
  }
  else {
    currentmap(CurrentMapName,MAX_MAP_LENGTH);    
    snprintf(Text, MAX_TEXT_LENGTH, "Time remaining on %s: %i %s.", CurrentMapName,Seconds,Verb);    
    say(Text);
    say("The next map will be determined by a vote.");
  }
  return 0;
}

/* **************************************************************************** */
/* Support functions for manging the map list data structures */

/* Called during initialisation, or explicitly by an admin */
/* Reads maps.ini or mapcycle.txt if maps.ini doesn't exist into an g_AllMaps */
ReadMapList() {

  g_AllMapsSize=0;
  new MapFile[MAX_MAP_LENGTH];
  getstrvar("maps_file",MapFile,MAX_MAP_LENGTH);

  if (fileexists(MapFile)==0) {
    MapFile="mapcycle.txt";
  }

  new ReadCount=min(MAX_MAPS,filesize(MapFile,1)+1);
  new i;
  for(i=0;i<ReadCount;i++) {
    /* File line numbers are indexed from 1, not 0, hence the +1 */
    readfile(MapFile,g_AllMaps[g_AllMapsSize],i,MAX_MAP_LENGTH);

    if (g_AllMaps[g_AllMapsSize][0] != '/') {
      /* Remove whitespace and ignore blank lines */
      strtrim(g_AllMaps[g_AllMapsSize]," ^t",2);
      
      GetShortName(g_AllMaps[g_AllMapsSize],g_AllMapsShort[g_AllMapsSize]);
      
      if (strlen(g_AllMaps[g_AllMapsSize])>0) {
	g_AllMapsSize++;
      }
    }
  }

  return 0;
}


ReportNominations(UserName[]) {
  new i=0;
  new msg[MAX_TEXT_LENGTH]="";
  new linelen=0;

  /*Stop the timer repeating this report too early */
  if (g_Nominators[0] == 0) {
    BuildLine(msg,"No maps have been nominated for the next vote.",0,linelen);
  }
  else {
    BuildLine(msg,"Maps nominated for the next vote: ",0,linelen);
    i=0;
    if (g_Nominators[1] != 0) {
      while (g_Nominators[i+2] >0) {
	BuildLine(msg,g_NominatedMaps[i],2,linelen);
	strcat(msg,", ",MAX_TEXT_LENGTH);
	i++;
      }
      BuildLine(msg,g_NominatedMaps[i],5,linelen);
      i++;
      strcat(msg," and ",MAX_TEXT_LENGTH);
    }
    BuildLine(msg,g_NominatedMaps[i],1,linelen);
    strcat(msg,".",MAX_TEXT_LENGTH);
  }

  if (strlen(UserName) == 0) {
    if ((g_MessageLocation & FLAG_NOMINATION) > 0) {
      BuildLine(msg,"^nTo nominate a map, say: nominate MAPNAME",0,linelen);
      SayAll(msg,FLAG_NOMINATION,15,255,0,255);    
    }
    else {
      SayAll(msg,FLAG_NOMINATION,15,255,0,255);
      SayAll("To nominate a map, say: nominate MAPNAME",FLAG_NOMINATION,15,255,0,255);
    }
  }
  else {
    BuildLine(msg,"^nTo nominate a map, say: nominate MAPNAME",0,linelen);
    messageex(UserName,msg,4);
  }
}


ReportMapsAvailable(UserName[]) {
  new i=0;
  new msg[MAX_TEXT_LENGTH]="";
  new linelen=0;
  new listlen=3;

  /*Stop the timer repeating this report too early */
  BuildLine(msg,"Maps ",0,linelen);
  for (i=0;i<listlen;i++) {
    if (i<listlen-2) {
      BuildLine(msg,g_AllMaps[g_InfoPosition],2,linelen);
      strcat(msg,", ",MAX_TEXT_LENGTH);
    }
    else {
      if (i==listlen-2) {
	BuildLine(msg,g_AllMaps[g_InfoPosition],5,linelen);
	strcat(msg," and ",MAX_TEXT_LENGTH);
      }
      else {
	BuildLine(msg,g_AllMaps[g_InfoPosition],0,linelen);
      }
    }

    g_InfoPosition++;
    if (g_InfoPosition == g_AllMapsSize) {
      g_InfoPosition = 0;
    }
  }

  BuildLine(msg," are available.",0,linelen);
  if (strlen(UserName)) {
    BuildLine(msg,"^nType bb_maps in the console for a full list",0,linelen);
    messageex(UserName,msg,4);
  }
  else {
    if ((g_MessageLocation & FLAG_AVAILABLEMAPS)>0) {
      BuildLine(msg,"^nType bb_maps in the console for a full list",0,linelen);
      SayAll(msg,FLAG_AVAILABLEMAPS,15,0,255,0);    
    }
    else {
      SayAll(msg,FLAG_AVAILABLEMAPS,15,0,255,0);
      SayAll("Type bb_maps in the console for a full list",FLAG_AVAILABLEMAPS,15,0,255,0);
    }
  }
}

ShowMapList() {
  new i=0;
  new msg[MAX_TEXT_LENGTH];

  while (i<g_AllMapsSize) {
    if (i+3<g_AllMapsSize) {
      snprintf(msg,MAX_TEXT_LENGTH,"%s, %s, %s, %s",g_AllMaps[i],g_AllMaps[i+1],g_AllMaps[i+2],g_AllMaps[i+3]);
    } else if (i+2<g_AllMapsSize) {
      snprintf(msg,MAX_TEXT_LENGTH,"%s, %s, %s",g_AllMaps[i],g_AllMaps[i+1],g_AllMaps[i+2]);
    } else if (i+1<g_AllMapsSize) {
      snprintf(msg,MAX_TEXT_LENGTH,"%s, %s",g_AllMaps[i],g_AllMaps[i+1]);
    } else {
      snprintf(msg,MAX_TEXT_LENGTH,"%s",g_AllMaps[i]);
    }
    selfmessage(msg);
    i=i+4;
  }
}


/* Builds up a message with no line longer than 78 characters
 * suitable for use with centresay */
BuildLine(msg[],txt[],extra,&linelen) {
  new txtlen = strlen(txt);
  if (strlen(txt) + extra + linelen >=78) {
    strcat(msg,"^n",MAX_TEXT_LENGTH);
    linelen=0;
  }
  strcat(msg,txt,MAX_TEXT_LENGTH);
  linelen=linelen + txtlen + extra;
}

/* Finds a slot in the nominations menu for a particular users nomination */
LookupFreeSlot(UserIndex) {
  new i;
  new j;

  /* First attempt to find an unused nomination slot */
  for(i=0;i<g_ThisMenuSize;i++) {
    if (g_Nominators[i] == 0) {
      return i;
    }
  }

  /* No free slots, is there a previous nomination from the user to replace? */
  for(i=g_ThisMenuSize-1;i>=0;i--) {
    if (g_Nominators[i] == UserIndex) {
      return i;
    }
  }

  /* OK, how about a previous user who has nominated more than one? */
  for(i=g_ThisMenuSize-1;i>=0;i--) {
    for(j=i;j<g_ThisMenuSize;j++) {
      if (g_Nominators[i] == g_Nominators[j]) {
	return j;
      }
    }
  }

  /* No free slots */
  return -1;

}

/* Checks if a name is in maps.ini */
LookupMap(MapName[]) {

  /* Remove an .bsp or other extension */
  new pos = strchr(MapName,'.');
  if (pos >0) {
    MapName[pos] = NULL_CHAR;
  }

  /* Check for an exact match */
  new i;
  for(i=0;i<g_AllMapsSize;i++) {
    if (strcasecmp(g_AllMaps[i],MapName)==0) {
      return i;
    }
  }

  /* Check for a match ignoring characters before the _ */
  for(i=0;i<g_AllMapsSize;i++) {
    if (strcasecmp(g_AllMapsShort[i],MapName)==0) {
      strcpy(MapName,g_AllMaps[i],MAX_MAP_LENGTH);
      return i;
    }
  }
  return -1;
}

/* Checks is a aname is already in the nominations menu */
IsNominated(MapName[]) {
  new i;
  for(i=0;i<g_ThisMenuSize;i++) {
    if (strcasecmp(g_NominatedMaps[i],MapName)==0) {
      return 1;
    }
  }
  return 0;
}

/* Checks is a name is in the bottom BannedMap places of g_AllMaps
 * and therefore is unavailable for nomination */
IsBanned(MapName[]) {

  new Current[MAX_MAP_LENGTH];
  currentmap(Current,MAX_MAP_LENGTH);
 
  if (strcasecmp(MapName,Current)==0) {
    return 1;
  }

  new i;
  for(i=g_AllMapsSize-g_BanMaps;i<g_AllMapsSize;i++) {
    if (streq(g_AllMaps[i],MapName)==1) {
      return 1;
    }
  }
  return 0;
}

MoveMapToEnd(MapName[],AllowVote) {
  new i = LookupMap(MapName);
  /* Shouldn't be able to happen, but just in case */
  if (i==-1) {
    return 0;
  }

  new offset=0;
  if (AllowVote == 1) {
	offset = g_BanMaps;
  }	

  while (i<(g_AllMapsSize-offset-1)) {
    strcpy(g_AllMaps[i],g_AllMaps[i+1],MAX_MAP_LENGTH);
    strcpy(g_AllMapsShort[i],g_AllMapsShort[i+1],MAX_MAP_LENGTH);
    i++;
  }
  strcpy(g_AllMaps[g_AllMapsSize-offset-1],MapName,MAX_MAP_LENGTH);
  GetShortName(MapName,g_AllMapsShort[g_AllMapsSize-offset-1]);
  return 0;
}

SaveMapList() {
  new i=0;
  new MapFile[MAX_MAP_LENGTH];
  getstrvar("maps_file",MapFile,MAX_MAP_LENGTH);

  /* Re-Write maps.ini with the new map order */
  if (fileexists(MapFile)==0) {
    MapFile = "mapcycle.txt";
  }
  resetfile(MapFile);


  for(i=0;i<g_AllMapsSize;i++) {
    writefile(MapFile,g_AllMaps[i]);
  }
  
  return 1;
}

public execute_all(a[]) {
  new userid = 0;
  new wonid = 0;
  new teamid = 0;
  new username[200];
  new i=0;
  new x = 0;
    
  x = maxplayercount();
  for(i=1; i<=x; i=i+1) {
    strinit(username);
    if(playerinfo(i, username, 200, userid, wonid, teamid)==1) {
      execclient(username, a);
    }
  }
}


SetMapTime(mins) {
  new ExecCommand[MAX_COMMAND_LENGTH];
  snprintf(ExecCommand, MAX_DATA_LENGTH, "mp_timelimit %i", mins/60);
  exec(ExecCommand); 
  return 0;
}


GetShortName(fullname[],shortname[]) {
  new pos = strchr(fullname,'_');
  if (pos>0) {
    pos = pos +1;
  }
  else {
    pos = 0;
  }
  
  new dpos=0;
  while (fullname[pos] != NULL_CHAR) {
    shortname[dpos]=fullname[pos];
    pos++;
    dpos++;
  }
  shortname[dpos] = NULL_CHAR;
}

SayAll(Msg[],Flag,Duration,Red,Green,Blue) {
  
  if ((Flag & g_Verbosity) > 0) {
    if ((Flag & g_MessageLocation) > 0) {
    	centersay(Msg,Duration,Red,Green,Blue);
    }
    else {
    	say(Msg);
    }
  }
}

readvaultnum(name[],&output,defaultval,minimum,maximum) {
  /* Read saved setting from vault */
  new Buffer[10]="";
  get_vaultdata(name,Buffer,10);
  if (strlen(Buffer)>0) {
    output = strtonum(Buffer);
    if ((output <minimum) || (output > maximum)) {
      output = defaultval;
    }
    return 1;
  }
  return 0;

}

writevaultnum(name[],value) {
  /* write number to vault */
  new Buffer[10];
  numtostr(value,Buffer);
  set_vaultdata(name,Buffer);
  return 0;
}

readHLnum(HLData,&output,defaultval,minimum,maximum) {
  /* Read setting from halflife string */
  new Data[MAX_DATA_LENGTH];
  convert_string(HLData,Data,MAX_DATA_LENGTH);
  if (strlen(Data) > 0) {
    output = strtonum(Data);
    if ((output <minimum) || (output > maximum)) {
      output = defaultval;
    }
    return 1;
  }
  return 0;
}

ShowLine(msg[],value) {
  new data[MAX_TEXT_LENGTH];
  snprintf(data,MAX_TEXT_LENGTH,msg,value);
  selfmessage(data);
}
